from ..types.na import NA
from ..types.earnings import Earnings

actual = Earnings("actual")
estimate = Earnings("estimate")
standardized = Earnings("standardized")

future_eps = NA(float)
future_time = NA(int)
future_revenue = NA(float)
future_period_end_time = NA(int)
