from ..types.pivotpointtype import PivotPointType

traditional = PivotPointType("Traditional")
fibonacci = PivotPointType("Fibonacci")
woodie = PivotPointType("Woodie")
classic = PivotPointType("Classic")
dm = PivotPointType("DM")
camarilla = PivotPointType("Camarilla")
